from dataclasses import dataclass
from typing import Dict, List, Optional

from now.constants import Apps, DatasetTypes, Qualities


@dataclass
class UserInput:
    app: Optional[Apps] = None

    # data related
    data: Optional[str] = None
    is_custom_dataset: Optional[bool] = None

    custom_dataset_type: Optional[DatasetTypes] = None
    dataset_name: Optional[str] = None
    dataset_url: Optional[str] = None
    dataset_path: Optional[str] = None
    # AWS related
    aws_access_key_id: Optional[str] = None
    aws_secret_access_key: Optional[str] = None
    aws_region_name: Optional[str] = None

    # model related
    quality: Optional[Qualities] = Qualities.MEDIUM

    # cluster related
    cluster: Optional[str] = None
    create_new_cluster: Optional[bool] = False
    deployment_type: Optional[str] = None
    secured: Optional[bool] = False
    jwt: Optional[Dict[str, str]] = None
    admin_emails: Optional[List[str]] = None
    user_emails: Optional[List[str]] = None
    additional_user: Optional[bool] = False



from fastapi import APIRouter

from deployment.bff.app.v1.models.admin import UpdateEmailsRequestModel
from deployment.bff.app.v1.routers.helper import jina_client_post

router = APIRouter()


# Index
@router.post(
    "/admin/updateUserEmails",
    summary='update user emails during runtime',
)
def update_user_email(data: UpdateEmailsRequestModel):
    """
    Append the list of image data to the indexer. Each image data should be
    `base64` encoded using human-readable characters - `utf-8`.
    """
    jina_client_post(
        host=data.host,
        port=data.port,
        inputs=[],
        endpoint='/admin/updateEmails',
        parameters={'jwt': data.jwt},
        target_executor=r'\Asecurity_check\Z',
    )



import inspect
import logging
import time
from contextlib import contextmanager
from datetime import timedelta
from functools import wraps
from typing import Callable

from fastapi import HTTPException

logger = logging.getLogger(__name__)


@contextmanager
def _count_elapsed(func: Callable):
    """
    A private context manager used for decoupling the timing logic in the timed
    decorator that comes in asyncio and non-asyncio variants.
    """
    start = time.time()
    yield
    elapsed = str(timedelta(seconds=(time.time() - start)))
    logger.debug(f'Execution of .{func.__name__}() took {elapsed}')


def timed(func: Callable):
    """
    Decorator to count and log elapsed time of a function call.
    """

    @wraps(func)
    def surround(*args, **kwargs):
        with _count_elapsed(func):
            return func(*args, **kwargs)

    return surround


def async_timed(func: Callable):
    """
    Decorator to count and log elapsed time of an async function call.
    """

    @wraps(func)
    async def surround(*args, **kwargs):
        with _count_elapsed(func):
            return await func(*args, **kwargs)

    return surround


def api_method(func: Callable):
    """Decorator to log api call info and handle exceptions."""

    @wraps(func)
    def surround(*args, **kwargs):
        func_name = inspect.getmodule(func).__name__ + f':{func.__name__}'
        logger.info(f'--- Calling api method {func_name} ...')
        try:
            response = func(*args, **kwargs)
        except HTTPException as exc:
            raise exc
        except Exception as exc:
            logger.error(
                f'An error occurred while running {func.__name__}(), '
                f'details: {str(exc)}'
            )
            logger.exception(exc)
            raise HTTPException(status_code=500, detail='Unknown error')
        return response

    return surround
